//  dithered palette demo
//
//  (c) 1993 By Jeffrey M. Cogswell
//  Windows Tech Journal
//  May, 1993
//
//  Feel free to implement pieces of this program into your own applications.
//
//  This program creates a palette of two custom colors
//  and demonstrates a dithered gradient of the two colors.
//  For details on the construction of the dither matrix
//  and lots of other great graphics information, see
//  _Computer Graphics: Principles and Practice_ by
//  Foley, vanDam, Feiner, and Huges, 2nd Edition,
//  Part of Addison-Wesley's "The Systems Programming Series". 
//
//



#include <windows.h>

long FAR PASCAL WndProc (HWND hWnd, WORD iMessage, WORD wParam, LONG lParam);
int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpszCmdParam, int nCmdShow)
{
static HWND hWnd;
MSG Message;
WNDCLASS WndClass;

if (!hPrevInstance)
	{
	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	WndClass.hbrBackground = GetStockObject(WHITE_BRUSH);
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	WndClass.hIcon = LoadIcon (NULL, IDI_APPLICATION);
	WndClass.hInstance = hInstance;
	WndClass.lpfnWndProc = (WNDPROC)WndProc;
	WndClass.lpszClassName = "DITHER";
	WndClass.lpszMenuName = NULL;
	WndClass.style = CS_HREDRAW | CS_VREDRAW;

	RegisterClass (&WndClass);
	}

hWnd = CreateWindow ("DITHER",
	"Dither Example Example",
	WS_OVERLAPPEDWINDOW | WS_SYSMENU,
	0,
	0,
	GetSystemMetrics(SM_CXSCREEN),
	GetSystemMetrics(SM_CYSCREEN),
	NULL,
	NULL,
	hInstance,
	NULL);

ShowWindow (hWnd, nCmdShow);
while (GetMessage (&Message, 0, 0, 0))
	{
	TranslateMessage(&Message);
	DispatchMessage(&Message);
	}
return Message.wParam;
}

/************************************************/
/*          Window Procedure :  WndProc         */
/************************************************/

long FAR PASCAL WndProc (HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{

/**** Device Context variables ****/
HDC hDC, hDC2;
static HBITMAP hbmp[65];
static HBRUSH hbrPrevious, hbr[65];
static HPEN hpen;
static int DitherMat2[2][2] =
    {0,  2,
     3,  1};
static int DitherMat4[4][4] =
    {0,  8,  2, 10,
    12,  4, 14,  6,
     3, 11,  1,  9,
    15,  7, 13,  5};
static int DitherMatrix[8][8] =
    {0, 32,  8, 40,  2, 34, 10, 42,
    48, 16, 56, 24, 50, 18, 58, 26,
    12, 44,  4, 36, 14, 46,  6, 38,
    60, 28, 52, 20, 62, 30, 54, 22,
     3, 35, 11, 43,  1, 33,  9, 41,
    51, 19, 59, 27, 49, 17, 57, 25,
    15, 47,  7, 39, 13, 45,  5, 37,
    63, 31, 55, 23, 61, 29, 53, 21};
static int DitherSize, NumIntensities;  //NumIntensities is always DitherSize*DitherSize+1.
int i,x,y,dx,dy,xpos,ypos;
PAINTSTRUCT PtStr;
RECT rc;
int Bits[8];

static HPALETTE hPal, hOldPal;          // Our very own palette and the previous one.
static HLOCAL hLocal;			        // Handle to the memory allocated for the logpalette
static NPLOGPALETTE npLogPal;		    // Pointer to the logpalette after the memory is locked
PALETTEENTRY PalEnt;                    // Single entry for dumping purposes.


switch (iMessage)
	{
    case WM_CREATE:
        DitherSize = 8;   // This is selectable between 2, 4, and 8.
        NumIntensities = DitherSize*DitherSize + 1;


        //Copy in the proper dither matrix.
        for (x=0; x<DitherSize; x++)
            for (y=0; y<DitherSize; y++)
            {
                if (DitherSize == 2) DitherMatrix[x][y] = DitherMat2[x][y];
                else if (DitherSize == 4) DitherMatrix[x][y] = DitherMat4[x][y];
            }
            

        //Create the dithered bitmap patterns.  Since we're dithering
        //two colors each time, we can create a monochrome bitmap
        //and set the colors later using SetBkColor and SetTextColor.

        //See accompanying text file for an explanation
        //of this rather cryptic algorithm.
        for (i=0;i<NumIntensities;i++)
        {
            for (y=0; y<DitherSize; y++)
            {
                Bits[y] = 0;
                for (x=0; x<DitherSize; x++)
                    if (DitherMatrix[x][y] < i)
                    {
                        if (DitherSize == 2)
                        {
                            Bits[y] |= ( 1<<x | 1<<(x+2) | 1<<(x+4) | 1<<(x+6) );
                            Bits[y+2] = Bits[y];
                            Bits[y+4] = Bits[y];
                            Bits[y+6] = Bits[y];
                        }
                        else if (DitherSize == 4)
                        {
                            Bits[y] |= ( 1<<x | 1<<(x+4) );
                            Bits[y+4] = Bits[y];
                        }
                        else if (DitherSize == 8)
                        {
                            Bits[y] |= 1<<x;
                        }
                    }
                    else
                    {
                        if (DitherSize == 2)
                        {
                            Bits[y] &= ~( 1<<x | 1<<(x+2) | 1<<(x+4) | 1<<(x+6) );
                            Bits[y+2] = Bits[y];
                            Bits[y+4] = Bits[y];
                            Bits[y+6] = Bits[y];
                        }
                        else if (DitherSize == 4)
                        {
                            Bits[y] &= ~( 1<<x | 1<<(x+4) );
                            Bits[y+4] = Bits[y];
                        }
                        else if (DitherSize == 8)
                        {
                            Bits[y] &= ~(1<<x);
                        }
                    }
            }
            hbmp[i] = CreateBitmap(8,8,1,1,Bits);
        }

        //create the brushes based on these bitmaps
        for (i=0; i<NumIntensities; i++)
            hbr[i] = CreatePatternBrush(hbmp[i]);

        //create a single null pen
        hpen = (HPEN)GetStockObject(NULL_PEN);


        //Create a palette of five colors.
        hLocal = LocalAlloc(LPTR, sizeof(LOGPALETTE) + 2 * sizeof(PALETTEENTRY));
        if (!hLocal)
        {
            MessageBox (NULL, "Couldn't Allocate Memory!", "BluePal", MB_OK);
            PostQuitMessage(0);
            return (0);
        }
        npLogPal = (NPLOGPALETTE) LocalLock(hLocal);
        npLogPal->palVersion = 0x300;
        npLogPal->palNumEntries = 2;

        // A couple of random colors
        npLogPal->palPalEntry[0].peBlue  = 200;
        npLogPal->palPalEntry[0].peRed   = 0;
        npLogPal->palPalEntry[0].peGreen = 0;
        npLogPal->palPalEntry[0].peFlags = 0;

        npLogPal->palPalEntry[1].peBlue  = 0;
        npLogPal->palPalEntry[1].peRed   = 150;
        npLogPal->palPalEntry[1].peGreen = 0;
        npLogPal->palPalEntry[1].peFlags = 0;

        hPal = CreatePalette((LPLOGPALETTE)npLogPal);
        if (!hPal) MessageBox (NULL, "Can't create palette!", "BluePal", MB_OK);
        LocalUnlock (hLocal);
        LocalFree (hLocal);

        return 0;

    case WM_PAINT:
 		hDC = BeginPaint(hWnd, &PtStr);
        SelectObject(hDC,hpen);

        hOldPal = SelectPalette(hDC, hPal, FALSE);
        if (!hOldPal)
        {
            MessageBox (NULL, "Couldn't Select Palette!", "BluePal", MB_OK);
        }
        else
        {
            RealizePalette(hDC);
        }
        hbrPrevious=SelectObject(hDC,hbr[0]); // this will get set twice, actually.

        GetClientRect(hWnd,&rc);
        dx = rc.right / NumIntensities;
        dy = rc.bottom / NumIntensities;
        xpos = 0;
        ypos = 0;
        SetTextColor(hDC,PALETTEINDEX(0));
        SetBkColor(hDC,PALETTEINDEX(1));
        xpos = 0;
        for (x=0; x<NumIntensities; x++)
        {
            SelectObject(hDC,hbr[x]);
            Rectangle (hDC,xpos,ypos,rc.right+1,ypos+dy+1);
            Rectangle (hDC,xpos,ypos,xpos+dx+1,rc.bottom+1);
            ypos += dy;
            xpos += dx;
        }
        //Fill in the last little area
        Rectangle (hDC,xpos,ypos,rc.right+1,rc.bottom+1);

        SelectObject(hDC, hbrPrevious);
        SelectPalette(hDC, hOldPal, FALSE);
        EndPaint(hWnd, &PtStr);
 		return (0);

	case WM_DESTROY:
        for (i=0; i<NumIntensities; i++)
        {
            DeleteObject(hbr[i]);
            DeleteObject(hbmp[i]);
        }
        DeleteObject(hpen);
        DeleteObject (hPal);
        PostQuitMessage(0);
        return (0);
	default:
        return (DefWindowProc (hWnd, iMessage, wParam, lParam));
	}
}
