//--------------------------------------------------------------------------
//
//  OBJCALLB.H
//
//  Object Thunk functions
//
//  These functions allow you to use instance specific callbacks for
//  classes.  Since callbacks are required to be static members of a
//  class, there is no "this" pointer.  Therefore when a callback gets
//  called, there is no knowledge of which instance of the class
//  is important.
//
//  MakeObjectThunk creates an instance-specific thunk for your
//  static member function.  Pass MakeObjectThunk a pointer to your
//  static class member your this pointer.  The returned pointer can be
//  used as an instance-specific alias for your static member function.
//  When your static member gets called, ES:BX will contain the this
//  pointer.  Use the following notation to retrieve it (under
//  Borland C++):
//
//  PThis = (... *)MAKELP(_ES,_BX);  // ES:BX set from the thunk
//
//  Special Note: You must compile your program using "Smart Callbacks"
//  (under Borland C++ use the -WS command line option. Under
//  MS C/C++ 7.0 use the /Ga option or _loadds keyword.)
//
//--------------------------------------------------------------------------


#ifndef __OBJCALLB_H
#define __OBJCALLB_H

#ifndef __WINDOWS_H
#include <windows.h>
#endif

//
//  Allocates an object callback
//
FARPROC WINAPI MakeObjectThunk(FARPROC CallBack, void far *fpObject);

//
//  Frees an object callback
//
void WINAPI FreeObjectThunk(FARPROC inst);

#endif
